
/*
 * GUI generates a set #defines per configured task to 
 * condition this templete file.
 */

/**********************************************************
 *
 * Required #defines:
 * ------------------
 * TASKSETUP_NAME:
 *   TaskSetup function name, set to TaskSetup_<TASK_NAME>
 * TASK_API:
 *   task API defined in dma_image.h
 * MAX_BD:
 *   <=0 : non-BD task
 *   else: number of BD in BD table
 * BD_FLAG:
 *   0   : no flag implemented for BD
 *   else: flags can be passed on a per BD basis
 * MISALIGNED:
 *   0   : task API supports Bytes%IncrBytes==0
 *   else: task API supports any parameter settings
 * AUTO_START:
 *   <-1       : do not start a task after task completion
 *   -1        : auto start the task after task completion
 *   <MAX_TASKS: auto start task with the TaskID = AUTO_START
 *    else     : do not start a task after task completion
 * INITIATOR_DATA:
 *   <0  : runtime configurable
 *   else: assume INITATOR_DATA equal hard-coded task initiator
 * SIZE_SRC: (needs to be consistent with Task API)
 *   SZ_FLEX  : Task API TYPE_SRC = flex, SzSrc defines size
 *   SZ_UINT8 : Task API TYPE_SRC = char
 *   SZ_UINT16: Task API TASK_SRC = short
 *   SZ_UINT32: Task API TASK_SRC = int
 * INCR_TYPE_SRC:
 *   0   : FIFO address, do not implement data pointer
 *   1   : automatic, set INCR_SRC based on SzSrc parameter
 *   2   : runtime, set INCR_SRC to IncrSrc parameter
 *   else: used hard-coded INCR_SRC
 * INCR_SRC:
 *   INCR_TYPE_SRC=0: force INCR_SRC=0
 *   else           : use for src pointer increment
 * SIZE_DST: (needs to be consistent with Task API)
 *   SZ_FLEX  : Task API TYPE_DST = flex, SzDst defines size
 *   SZ_UINT8 : Task API TYPE_DST = char
 *   SZ_UINT16: Task API TASK_DST = short
 *   SZ_UINT32: Task API TASK_DST = int
 * INCR_TYPE_DST:
 *   0   : FIFO address, do not implement data pointer
 *   1   : automatic, set INCR_DST based on SzDst parameter
 *   2   : runtime, set INCR_DST to IncrDst parameter 
 *   else: used hard-coded INCR_DST
 * INCR_DST:
 *   INCR_TYPE_DST=0: force INCR_DST=0
 *   else           : use for dst pointer increment
 *
 * Optional #defines:
 * ------------------
 * MAX_TASKS:
 *   1   : #define MAX_TASKS>0
 *   else: 16
 * ITERATIONS:
 *   1   : #define ITERATIONS>0
 *   else: 1
 * INCR_BYTES:
 *   This macro is defined based on following priority:
 *   1   : INCR_SRC != 0
 *   2   : DST_TYPE != 0
 *   3   : #defined INCR_BYTES<0   
 *   else: -4 (SZ_UINT32)
 * DEBUG_BESTCOMM_API:
 *   >0  : print basic debug messages
 *   >=10: also print C-API interface variables
 *   >=20: also print task API interface variables
 *   else: do nothing
 *
 **********************************************************/

#if NOT_LINUX_KERNEL
#include <stdlib.h>
#endif

#include "dma_image.h"
//#include "dma_image.capi.h"

#include "bestcomm_api.h"
#include "tasksetup_bdtable.h"

#include "bestcomm_api_mem.h"
#include "bestcomm_cntrl.h"

#if (INCR_TYPE_SRC==0)   /* FIFO address, no data pointer */
 #undef  INCR_SRC
 #define INCR_SRC     0
#endif

#if (INCR_TYPE_DST==0)   /* FIFO address, no data pointer */
 #undef  INCR_DST
 #define INCR_DST     0
#endif

#ifndef  MAX_TASKS
 #define MAX_TASKS   16
#else
 #if (MAX_TASKS<=0)
  #undef  MAX_TASKS
  #define MAX_TASKS  16
 #endif
#endif

#ifndef  ITERATIONS
 #define ITERATIONS   1
#else
 #if (ITERATIONS<=0)
  #undef  ITERATIONS
  #define ITERATIONS  1
 #endif
#endif

#ifndef   INCR_BYTES
  #define INCR_BYTES -4
#else
 #if (INCR_BYTES>=0)
  #undef  INCR_BYTES
  #define INCR_BYTES -4
 #endif
#endif

#ifdef TASK_API

TaskId TASKSETUP_NAME (TASK_API            *TaskAPI,
                       TaskSetupParamSet_t *TaskSetupParams)
{

   TaskId TaskNum = (TaskId)SDMA_TASKNUM_EXT(TaskAPI->TaskNum);
   uint32 Status  = INCR_SRC; /* Tx:Status=0, Rx:Status!=0 */
   uint8  NumPtr  = 0;

#if (DEBUG_BESTCOMM_API>0)
   printf("\nBestComm API Debug Display Mode Enabled\n\n");
   printf("TaskSetup: TaskID=%d\n", TaskNum);
   if (Status!=0) printf("TaskSetup: Rx task\n");
   else           printf("TaskSetup: Tx or DP task\n");
#endif

#if (MAX_BD>0)   /* Buffer Descriptors */

 #if (INCR_TYPE_SRC!=0)
   ++NumPtr;
 #endif
 #if (INCR_TYPE_DST!=0)
   ++NumPtr;
 #endif

 #if (DEBUG_BESTCOMM_API>0)
   printf("TaskSetup: Using %d buffer descriptors, each with %d data pointers\n", MAX_BD, NumPtr);
 #endif

   /* Allocate BD table SRAM storage, 
    * and pass addresses to task API */
   TaskSetup_BDTable(TaskAPI->BDTableBase,
                     TaskAPI->BDTableLast,
                     TaskNum,
                     TaskSetupParams->NumBD,
                     MAX_BD, NumPtr,
		     BD_FLAG, Status);

 #if BD_FLAG

  #if (DEBUG_BESTCOMM_API>0)
   printf("TaskSetup: Buffer descriptor flags are enabled\n");
  #endif

   /* always assume 2nd to last DRD */
   *TaskAPI->AddrDRD      = (uint32)TaskAPI->DRD[(TaskAPI->NumDRD-2)];
 #endif

#else   /* No Buffer Descriptors */

  #if (DEBUG_BESTCOMM_API>0)
   printf("TaskSetup: Task will complete %d iterations before disabling\n");
  #endif

   *TaskAPI->IterExtra = (uint32)(ITERATIONS-1);

#endif

/* Setup auto start */
#if   (AUTO_START <= -2 )        /* do not start a task */
 #if (DEBUG_BESTCOMM_API>0)
   printf("TaskSetup: Auto task start disabled\n");
 #endif
   SDMA_TASK_CFG(SDMA_TCR, TaskNum, 0, TaskNum);
#elif (AUTO_START <= -1 )        /* restart task        */
 #if (DEBUG_BESTCOMM_API>0)
   printf("TaskSetup: Auto start task\n");
 #endif
   SDMA_TASK_CFG(SDMA_TCR, TaskNum, 1, TaskNum);
#elif (AUTO_START < MAX_TASKS)   /* start specific task */
 #if (DEBUG_BESTCOMM_API>0)
   printf("TaskSetup: Auto start task with TaskID=%d\n", AUTO_START);
 #endif
   SDMA_TASK_CFG(SDMA_TCR, TaskNum, 1, AUTO_START);
#else                            /* do not start a task */
 #if (DEBUG_BESTCOMM_API>0)
   printf("TaskSetup: Auto task start disabled\n");
 #endif
   SDMA_TASK_CFG(SDMA_TCR, TaskNum, 0, TaskNum);
#endif


#if (DEBUG_BESTCOMM_API>=10)
   printf("\nTaskSetup: C-API Parameter Settings Before:\n");
   printf("TaskSetup:   NumBD        = %d\n", TaskSetupParams->NumBD);
 #if (MAX_BD>0)
   printf("TaskSetup:   MaxBuf       = %d\n", TaskSetupParams->Size.MaxBuf);
 #else
   printf("TaskSetup:   NumBytes     = %d\n", TaskSetupParams->Size.NumBytes);
 #endif
   printf("TaskSetup:   Initiator    = %d\n", TaskSetupParams->Initiator);
   printf("TaskSetup:   StartAddrSrc = 0x%08X\n", TaskSetupParams->StartAddrSrc);
   printf("TaskSetup:   IncrSrc      = %d\n", TaskSetupParams->IncrSrc);
   printf("TaskSetup:   SzSrc        = %d\n", TaskSetupParams->IncrSrc);
   printf("TaskSetup:   StartAddrDst = 0x%08X\n", TaskSetupParams->StartAddrDst);
   printf("TaskSetup:   IncrDst      = %d\n", TaskSetupParams->IncrDst);
   printf("TaskSetup:   SzDst        = %d\n", TaskSetupParams->IncrDst);
#endif

#if (DEBUG_BESTCOMM_API>=20)
   printf("\nTaskSetup: Task-API Parameter Settings Before:\n");
 #if (MAX_BD>0)
   printf("TaskSetup:   BDTableBase  = 0x%08X\n", *TaskAPI->BDTableBase);
   printf("TaskSetup:   BDTableLast  = 0x%08X\n", *TaskAPI->BDTableLast);
  #if (BD_FLAG)
   printf("TaskSetup:   AddrDRD      = 0x%08X\n", *TaskAPI->AddrDRD);
  #endif
 #else
   printf("TaskSetup:   IterExtra    = %d\n", *TaskAPI->IterExtra);
  #if (INCR_TYPE_SRC==0)
   printf("TaskSetup:   AddrSrcFIFO  = 0x%08X\n", *TaskAPI->AddrSrcFIFO);
  #else
   printf("TaskSetup:   StartAddrSrc = 0x%08X\n", *TaskAPI->StartAddrSrc);
  #endif
  #if (INCR_TYPE_DST==0)
   printf("TaskSetup:   AddrDstFIFO  = 0x%08X\n", *TaskAPI->AddrDstFIFO);
  #else
   printf("TaskSetup:   StartAddrDst = 0x%08X\n", *TaskAPI->StartAddrDst);
  #endif
 #endif
 #if (INCR_TYPE_SRC!=0)
   printf("TaskSetup:   IncrSrc      = 0x%04X\n", *TaskAPI->IncrSrc);
  #if (MISALIGNED)
   printf("TaskSetup:   IncrSrcMA    = 0x%04X\n", *TaskAPI->IncrSrcMA);
  #endif
 #endif
 #if (INCR_TYPE_DST!=0)
   printf("TaskSetup:   IncrDst      = 0x%04X\n", *TaskAPI->IncrDst);
  #if (MISALIGNED)
   printf("TaskSetup:   IncrDstMA    = 0x%04X\n", *TaskAPI->IncrDstMA);
  #endif
 #endif
   printf("TaskSetup:   Bytes        = %d\n", *TaskAPI->Bytes);
   printf("TaskSetup:   IncrBytes    = %d\n", *TaskAPI->IncrBytes);
#endif


#if (INITIATOR_DATA<0)  /* runtime configurable */
   SDMA_SET_INIT(SDMA_TCR, TaskNum, TaskSetupParams->Initiator);
#else                   /* fixed */
   TaskSetupParams->Initiator = INITIATOR_DATA;
#endif

   *TaskAPI->Bytes = (uint32)TaskSetupParams->Size.MaxBuf;


#if (SIZE_SRC!=SZ_FLEX)  /* size fixed in task code */
   TaskSetupParams->SzSrc = SIZE_SRC;
#endif

#if    (INCR_TYPE_SRC==0)   /* no data pointer */
   TaskSetupParams->IncrSrc = (sint16)0;
   *TaskAPI->AddrSrcFIFO = (uint32)TaskSetupParams->StartAddrSrc;
#else

 #if   (INCR_TYPE_SRC==1)  /* automatic */
   if (TaskSetupParams->IncrSrc!=0) {
      TaskSetupParams->IncrSrc = (sint16)+TaskSetupParams->SzSrc;
   }
   else {
      TaskSetupParams->IncrSrc = (sint16)+TaskSetupParams->IncrSrc;
   }
 #elif (INCR_TYPE_SRC!=2)  /* hard-coded */
   TaskSetupParams->IncrSrc = (sint16)INCR_SRC;
 #endif
 *TaskAPI->IncrSrc = (sint16)TaskSetupParams->IncrSrc;

 #if (MAX_BD>0)    /* pointer in BD Table */
   /* pass back address of first BD */
   TaskSetupParams->StartAddrSrc = (uint32)TaskGetBD(TaskNum, 0);
 #else
   *TaskAPI->StartAddrSrc = (uint32)TaskSetupParams->StartAddrSrc;
 #endif

 #if MISALIGNED
   if      (TaskSetupParams->IncrSrc < 0) {
      *TaskAPI->IncrSrcMA = (sint16)-1;
   }
   else if (TaskSetupParams->IncrSrc > 0) {
      *TaskAPI->IncrSrcMA = (sint16)+1;
   }
   else {
      *TaskAPI->IncrSrcMA = (sint16)0;
   }
 #endif
#endif


#if (SIZE_DST!=SZ_FLEX)  /* size fixed in task code */
   TaskSetupParams->SzDst = SIZE_DST;
#endif

#if    (INCR_TYPE_DST==0)  /* no data pointer */
   TaskSetupParams->IncrDst = (sint16)0;
   *TaskAPI->AddrDstFIFO = (uint32)TaskSetupParams->StartAddrDst;
#else
 #if   (INCR_TYPE_DST==1)  /* automatic */
   if (TaskSetupParams->IncrDst!=0) {
      TaskSetupParams->IncrDst = (sint16)+TaskSetupParams->SzDst;
   }
   else {
      TaskSetupParams->IncrDst = (sint16)+TaskSetupParams->IncrDst;
   }
 #elif (INCR_TYPE_DST!=2)  /* hard-coded */
   TaskSetupParams->IncrDst = (sint16)INCR_DST;
 #endif
   *TaskAPI->IncrDst = (sint16)TaskSetupParams->IncrDst;

 #if (MAX_BD>0)
   /* pass back address of first BD */
   TaskSetupParams->StartAddrDst = (uint32)TaskGetBD(TaskNum, 0);
 #else
   *TaskAPI->StartAddrDst = (uint32)TaskSetupParams->StartAddrDst;
 #endif

 #if MISALIGNED
   if      (TaskSetupParams->IncrDst < 0) {
      *TaskAPI->IncrDstMA = (sint16)-1;
   }
   else if (TaskSetupParams->IncrDst > 0) {
      *TaskAPI->IncrDstMA = (sint16)+1;
   }
   else {
      *TaskAPI->IncrDstMA = (sint16)0;
   }
 #endif
#endif

/* always use macro, only affect code with #define TYPE_? flex */
   SDMA_SET_SIZE(SDMA_TASK_SIZE, TaskNum, TaskSetupParams->SzSrc, TaskSetupParams->SzDst);


   if      (TaskSetupParams->IncrSrc != 0) {
      *TaskAPI->IncrBytes = (sint16)-abs(TaskSetupParams->IncrSrc);
   }
   else if (TaskSetupParams->IncrDst != 0) {
      *TaskAPI->IncrBytes = (sint16)-abs(TaskSetupParams->IncrDst);
   }
   else {
      *TaskAPI->IncrBytes = (sint16)-abs(INCR_BYTES);
   }


#if (DEBUG_BESTCOMM_API>=10)
   printf("\nTaskSetup: C-API Parameter Settings After:\n");
   printf("TaskSetup:   NumBD        = %d\n", TaskSetupParams->NumBD);
 #if (MAX_BD>0)
   printf("TaskSetup:   MaxBuf       = %d\n", TaskSetupParams->Size.MaxBuf);
 #else
   printf("TaskSetup:   NumBytes     = %d\n", TaskSetupParams->Size.NumBytes);
 #endif
   printf("TaskSetup:   Initiator    = %d\n", TaskSetupParams->Initiator);
   printf("TaskSetup:   StartAddrSrc = 0x%08X\n", TaskSetupParams->StartAddrSrc);
   printf("TaskSetup:   IncrSrc      = %d\n", TaskSetupParams->IncrSrc);
   printf("TaskSetup:   SzSrc        = %d\n", TaskSetupParams->IncrSrc);
   printf("TaskSetup:   StartAddrDst = 0x%08X\n", TaskSetupParams->StartAddrDst);
   printf("TaskSetup:   IncrDst      = %d\n", TaskSetupParams->IncrDst);
   printf("TaskSetup:   SzDst        = %d\n", TaskSetupParams->IncrDst);
#endif

#if (DEBUG_BESTCOMM_API>=20)
   printf("\nTaskSetup: Task-API Parameter Settings Before:\n");
 #if (MAX_BD>0)
   printf("TaskSetup:   BDTableBase  = 0x%08X\n", *TaskAPI->BDTableBase);
   printf("TaskSetup:   BDTableLast  = 0x%08X\n", *TaskAPI->BDTableLast);
  #if (BD_FLAG)
   printf("TaskSetup:   AddrDRD      = 0x%08X\n", *TaskAPI->AddrDRD);
  #endif
 #else
   printf("TaskSetup:   IterExtra    = %d\n", *TaskAPI->IterExtra);
  #if (INCR_TYPE_SRC==0)
   printf("TaskSetup:   AddrSrcFIFO  = 0x%08X\n", *TaskAPI->AddrSrcFIFO);
  #else
   printf("TaskSetup:   StartAddrSrc = 0x%08X\n", *TaskAPI->StartAddrSrc);
  #endif
  #if (INCR_TYPE_DST==0)
   printf("TaskSetup:   AddrDstFIFO  = 0x%08X\n", *TaskAPI->AddrDstFIFO);
  #else
   printf("TaskSetup:   StartAddrDst = 0x%08X\n", *TaskAPI->StartAddrDst);
  #endif
 #endif
 #if (INCR_TYPE_SRC!=0)
   printf("TaskSetup:   IncrSrc      = 0x%04X\n", *TaskAPI->IncrSrc);
  #if (MISALIGNED)
   printf("TaskSetup:   IncrSrcMA    = 0x%04X\n", *TaskAPI->IncrSrcMA);
  #endif
 #endif
 #if (INCR_TYPE_DST!=0)
   printf("TaskSetup:   IncrDst      = 0x%04X\n", *TaskAPI->IncrDst);
  #if (MISALIGNED)
   printf("TaskSetup:   IncrDstMA    = 0x%04X\n", *TaskAPI->IncrDstMA);
  #endif
 #endif
   printf("TaskSetup:   Bytes        = %d\n", *TaskAPI->Bytes);
   printf("TaskSetup:   IncrBytes    = %d\n", *TaskAPI->IncrBytes);
#endif


   return TaskNum;
}

#endif  /* TASK_API defined */
